# # -*- coding: utf-8 -*-
# from pydrive import ...
# from enerpi.base import CONFIG, log, async_task

# TODO GDRIVE SYNC
# RECIPIENT = 'eugenio.panadero@gmail.com'
# EMAIL_ACCOUNT = 'enerpi.bot@gmail.com'
# GMAIL_APP_PASSWORD = 'qkdspbhmxouzrkvv'
