{% extends "layout_simple.html" %}
{% from 'monitor_tiles.html' import monitor_tiles, monitor_tiles_styles, monitor_tiles_scripts with context %}

{% block styles %}
    {{ monitor_tiles_styles() }}
{% endblock %}

{% block content %}
    {{ monitor_tiles(data_monitor, include_consumption) }}
{% endblock %}

{% block scripts %}
    {{ monitor_tiles_scripts(data_monitor) }}
{% endblock %}
